/* IBM DB2 - Scripting                                                                                                     */
/* Database: ReqPro                                                                                                        */
/* Creation 09/17/2009                                                                                                     */
/* Version: 1.00.0003                                                                                                      */

/* History:                                                                                                                */
/* DATE         DEVELOPER(S)           VERSION   DESCRIPTION                                                               */
/* 09/17/2009   Babar Abdul Qayyum     7.1       New set of tables, procedures and triggers to handle RRC syncronization   */
/*              Bill Betz                                                                                                  */
/*              Alex Chalini                                                                                               */

--    CREATE PROCEDURE RqGetRRCID (IN p_reqid INT, OUT RRCID VARCHAR(255))
--    DYNAMIC RESULT SETS 0
--    LANGUAGE SQL
--    READS SQL DATA
--    BEGIN
--       DECLARE charvalue varchar(255);
--       DECLARE inputV BLOB;
--    
--       SET charvalue = '';
--       SET RRCID     = '';
--    
--       SELECT RESOURCEOBJ INTO inputV FROM RQRESOURCES WHERE REQUIREMENTID=p_reqid and FLAGS=2;
--    
--       SET charvalue = CAST(inputV AS VARCHAR(255)) ;
--       SET RRCID = rtrim(SUBSTR(charvalue, (LOCATE('resources/', rtrim(charvalue))+10), 60));
--    
--    END @



/*

-- DROP OPBJECTS

ALTER TABLE RQTLOG DROP CONSTRAINT FK_RQTLOG_FK1;
ALTER TABLE RQTLOG DROP CONSTRAINT FK_RQTLOG_FK2;
ALTER TABLE RQTLOGCOLUMNS DROP CONSTRAINT FK_RQTLOGC_FK1;
ALTER TABLE RQTLOGOVERFLOW DROP CONSTRAINT FK_RQTLOGOVFW_FK1;
-- ALTER TABLE RQTLOGDOCUMENTS DROP CONSTRAINT FK_RQTLOGDCS_FK1;

-- DROP PROCEDURE RqGetRRCID;
DROP TABLE RQTLOGOBJECTTYPE;
DROP TABLE RQTLOGTRANSTYPE;
DROP TABLE RQTLOG;
DROP TABLE RQTLOGCOLUMNS;
DROP TABLE RQTLOGOVERFLOW;
-- DROP TABLE RQTLOGDOCUMENTS;
DROP TABLE RQTLOGMAP;
DROP TABLE RQTLOGPROPERTY;
DROP PROCEDURE RQINSERT_RQTLOG;
DROP PROCEDURE RQINSERT_RQTLOGCOLUMNS;
DROP PROCEDURE RQINSERT_RQTLOGOVERFLOW;
-- DROP PROCEDURE RQINSERT_RQTLOGDOCUMENTS;
DROP PROCEDURE RQINS_TLOGCOLUMNS_FIELDVALUES;
DROP PROCEDURE RQINS_TLOGCOLUMNS_LISTVALUES;
DROP PROCEDURE RQINS_TLOGCOLUMNS_REQ;
DROP PROCEDURE RQINSERT_LOG_REQUIREMENT;
-- DROP PROCEDURE MIGRATE_TLOGMAP;
DROP PROCEDURE RQDELETE_LOG_REQUIREMENT;
DROP PROCEDURE RQDELETE_RQTLOGMAP_ENTRY;
-- DROP PROCEDURE RQGET_TLOGMAP_SQL;
DROP VIEW RQTLOGV_SYNCUSR;
DROP TRIGGER RQREQHIST_TRIG_LOG;
-- DROP TRIGGER RQTLOGMAP_DTRIG;

DROP FUNCTION UDFBITOR ;
DROP FUNCTION UDFBITAND;


DROP PROCEDURE RQDEL_RQTLOGMAP;

*/



CREATE FUNCTION UDFBITAND (N1 Integer, N2 Integer)
 RETURNS Integer
 LANGUAGE SQL
 CONTAINS SQL
 NO EXTERNAL ACTION
 DETERMINISTIC
BEGIN ATOMIC
   DECLARE M1, M2, S Integer;
   DECLARE RetVal Integer DEFAULT 0;
    
   SET (M1, M2, S) = (N1, N2, 0);
   WHILE M1 > 0 AND M2 > 0 AND S < 32 DO
      SET RetVal = RetVal + MOD(M1,2)*MOD(M2,2)*power(2,S);
      SET (M1, M2, S) = (M1/2, M2/2, S+1);
   END WHILE;
    
   RETURN RetVal;
END @



CREATE FUNCTION UDFBITOR( x Integer, y Integer )
 RETURNS Integer
 LANGUAGE SQL
 CONTAINS SQL
 NO EXTERNAL ACTION
 DETERMINISTIC
BEGIN ATOMIC
    RETURN x + y - UDFBITAND(x,y);
END @


/****** Object:  Table RQTLOGOBJECTTYPE ******/

CREATE TABLE RQTLOGOBJECTTYPE(
   ID int NOT NULL,
   OBJECTTYPE varchar(20) NOT NULL,
   CONSTRAINT PK_RQTLOGOBJECTTYP PRIMARY KEY (ID)
) IN TABLE16K INDEX IN INDEX4K@



/****** Object:  Table RQTLOGTRANSTYPE ******/

CREATE TABLE RQTLOGTRANSTYPE(
   ID int NOT NULL,
   TRANSTYPE varchar(20) NOT NULL,
   CONSTRAINT PK_RQTLOGTRANSTYPE PRIMARY KEY (ID)
) IN TABLE16K INDEX IN INDEX4K@





/****** Object:  Table RQTLOG ******/

CREATE TABLE RQTLOG(
   ID DECIMAL (10) NOT NULL GENERATED BY DEFAULT AS IDENTITY (START WITH 1, INCREMENT BY 1, NO CACHE),
   PROJECTGUID varchar(40) NOT NULL,
   PROJECTMAPID varchar(40),
   USERID varchar(20) NOT NULL,
   TDATE timestamp NOT NULL,
   ISDOCUMENT smallint NOT NULL,
   RQTTRANSTYPEID int NOT NULL,
   RQTOBJECTTYPEID int NOT NULL,
   RQTOBJECTGUID varchar(40) NOT NULL,
   RQTEXTERNALID varchar(40),
 CONSTRAINT PK_RQTLOG PRIMARY KEY (ID)
) IN TABLE4K INDEX IN INDEX4K@



ALTER TABLE RQTLOG ADD CONSTRAINT FK_RQTLOG_FK1 FOREIGN KEY(RQTOBJECTTYPEID) REFERENCES RQTLOGOBJECTTYPE (ID)@

ALTER TABLE RQTLOG ADD CONSTRAINT FK_RQTLOG_FK2 FOREIGN KEY(RQTTRANSTYPEID) REFERENCES RQTLOGTRANSTYPE (ID)@







/****** Object:  Table RQTLOGCOLUMNS ******/

CREATE TABLE RQTLOGCOLUMNS(
   ID int NOT NULL GENERATED BY DEFAULT AS IDENTITY (START WITH 1, INCREMENT BY 1, NO CACHE),
   RQTLOGID int NOT NULL,
   TABLENAME varchar(150) NOT NULL,
   TID varchar(40) NOT NULL,
   COLUMNNAME varchar(200) NOT NULL,
   COLUMNVALUE varchar(4000),
   ISOVERFLOW smallint NOT NULL,
 CONSTRAINT PK_RQTLOGCOLUMNS PRIMARY KEY (ID) 
) IN TABLE16K INDEX IN INDEX4K@


ALTER TABLE RQTLOGCOLUMNS ADD CONSTRAINT FK_RQTLOGC_FK1 FOREIGN KEY(RQTLOGID) REFERENCES RQTLOG (ID) ON DELETE CASCADE@



/****** Object:  Table RQTLOGOVERFLOW ******/

CREATE TABLE RQTLOGOVERFLOW(
   RQTLOGCOLUMNSID int NOT NULL,
   OVERFLOWTEXT CLOB 
) IN TABLE16K INDEX IN INDEX4K@


ALTER TABLE RQTLOGOVERFLOW ADD CONSTRAINT FK_RQTLOGOVFW_FK1 FOREIGN KEY(RQTLOGCOLUMNSID) REFERENCES RQTLOGCOLUMNS (ID) ON DELETE CASCADE@


/****** Object:  Table RQTLOGDOCUMENTS ******/

--    CREATE TABLE RQTLOGDOCUMENTS(
--       RQTLOGID int NOT NULL,
--       RQDOCUMENTGUID varchar(40) NOT NULL,
--       RQDOCUMENTMAPID varchar(40),
--       VERSIONNUMBER varchar(50) NOT NULL,
--       RQDOCUMENT CLOB 
--    ) IN TABLE16K INDEX IN INDEX4K@

--    ALTER TABLE RQTLOGDOCUMENTS ADD CONSTRAINT FK_RQTLOGDCS_FK1 FOREIGN KEY(RQTLOGID) REFERENCES RQTLOG (ID) ON DELETE CASCADE@





/****** Object:  Table RQTLOGMAP ******/

CREATE TABLE RQTLOGMAP(
   ID int NOT NULL GENERATED BY DEFAULT AS IDENTITY (START WITH 1, INCREMENT BY 1, NO CACHE),
   OBJECTTYPE varchar(30) NOT NULL,
   OBJECTGUID varchar(200) NOT NULL,
   EXTERNALID varchar(40) NOT NULL,
   CONSTRAINT PK_RQTLOGMAP PRIMARY KEY (ID),
   CONSTRAINT UK_RQTLOGMAP UNIQUE ( OBJECTTYPE, OBJECTGUID )
) IN TABLE16K INDEX IN INDEX4K@






/****** Object:  Table RQTLOGPROPERTY ******/

CREATE TABLE RQTLOGPROPERTY(
   PROJECTGUID varchar(40) NOT NULL,
   KEYNAME varchar(100) NOT NULL,
   VALUE varchar(200) NOT NULL,
   CONSTRAINT PK_RQTLOGPROPERTY PRIMARY KEY (PROJECTGUID, KEYNAME)
) IN TABLE16K INDEX IN INDEX4K@





INSERT INTO RQTLOGTRANSTYPE VALUES (1, 'insert')@
INSERT INTO RQTLOGTRANSTYPE VALUES (2, 'update')@
INSERT INTO RQTLOGTRANSTYPE VALUES (3, 'delete')@
INSERT INTO RQTLOGOBJECTTYPE VALUES (1, 'project')@
INSERT INTO RQTLOGOBJECTTYPE VALUES (2, 'requirementtype')@
INSERT INTO RQTLOGOBJECTTYPE VALUES (3, 'attribute')@
INSERT INTO RQTLOGOBJECTTYPE VALUES (4, 'package')@
INSERT INTO RQTLOGOBJECTTYPE VALUES (5, 'document')@
INSERT INTO RQTLOGOBJECTTYPE VALUES (6, 'requirement')@



--    CREATE PROCEDURE RQGET_TLOGMAP_SQL
--    (
--       IN L_PROJECTGUID varchar(40),
--       IN L_TOPN int,
--       IN L_SCHEMANAME varchar(30),
--       OUT L_QUERY varchar(1000)
--    )
--    LANGUAGE SQL
--    BEGIN
--    	SET L_QUERY = 'SELECT RQTL.*, TransType.TRANSTYPE, ObjectType.OBJECTTYPE FROM ' || 
--    	               L_SCHEMANAME || '.RQTLOG RQTL, ' || L_SCHEMANAME || '.RQTLOGTRANSTYPE TransType, ' || L_SCHEMANAME || 
--    	               '.RQTLOGOBJECTTYPE ObjectType WHERE RQTL.PROJECTGUID=''' || L_PROJECTGUID || ''' AND RQTL.RQTTRANSTYPEID=TransType.ID AND RQTL.RQTOBJECTTYPEID=ObjectType.ID ORDER BY RQTL.ID';
	--    
--    	IF L_TOPN > 0 THEN
--    		SET L_QUERY = L_QUERY || ' FETCH FIRST ' || CAST(L_TOPN AS VARCHAR(5)) || ' ROWS ONLY OPTIMIZE FOR ' || CAST(L_TOPN AS VARCHAR(5)) || ' ROWS';
--    	END IF;
--    END @


/****** Object:  StoredProcedure RQINSERT_RQTLOG ******/
CREATE PROCEDURE RQINSERT_RQTLOG
(
   IN L_PROJECTGUID varchar(40),
   IN L_PROJECTMAPID varchar(40),
   IN L_USERID varchar(20),
   IN L_TDATE timestamp,
   IN L_ISDOCUMENT smallint,
   IN L_RQTTRANSTYPEID int,
   IN L_RQTOBJECTTYPEID int,
   IN L_RQTOBJECTGUID varchar(40),
   IN L_RQTEXTERNALID varchar(40),
   OUT L_LOGSID int
)
DYNAMIC RESULT SETS 0
LANGUAGE SQL
MODIFIES SQL DATA
BEGIN

   SET L_LOGSID=0;
   
   -- insert column values into RQTLOG table
   INSERT INTO RQTLOG (PROJECTGUID,PROJECTMAPID,USERID,TDATE,ISDOCUMENT,RQTTRANSTYPEID,RQTOBJECTTYPEID,RQTOBJECTGUID,RQTEXTERNALID)
   VALUES (L_PROJECTGUID,L_PROJECTMAPID,L_USERID,L_TDATE,L_ISDOCUMENT,L_RQTTRANSTYPEID,L_RQTOBJECTTYPEID,L_RQTOBJECTGUID,L_RQTEXTERNALID);
   
   SET L_LOGSID=IDENTITY_VAL_LOCAL();
END @






/****** Object:  StoredProcedure RQINSERT_RQTLOGCOLUMNS ******/
CREATE PROCEDURE RQINSERT_RQTLOGCOLUMNS
(
   IN L_RQTLOGID int,
   IN L_TABLENAME varchar(150),
   IN L_TID varchar(40),
   IN L_COLUMNNAME varchar(300),
   IN L_COLUMNVALUE varchar(32672),
   IN L_ISOVERFLOW smallint,
   OUT L_LOGSCOLUMNID DECIMAL(10)
)
DYNAMIC RESULT SETS 0
LANGUAGE SQL
MODIFIES SQL DATA
BEGIN

   SET L_LOGSCOLUMNID=0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   VALUES (L_RQTLOGID,L_TABLENAME,L_TID,L_COLUMNNAME,L_COLUMNVALUE,L_ISOVERFLOW);
   
   SET L_LOGSCOLUMNID=IDENTITY_VAL_LOCAL();
END @






/****** Object:  StoredProcedure RQINSERT_RQTLOGOVERFLOW ******/
CREATE PROCEDURE RQINSERT_RQTLOGOVERFLOW
(
   L_RQTLOGCOLUMNSID INT,
   L_OVERFLOWTEXT CLOB
)
DYNAMIC RESULT SETS 0
LANGUAGE SQL
MODIFIES SQL DATA
BEGIN

   -- insert column values into RQTLOG table
   INSERT INTO RQTLOGOVERFLOW (RQTLOGCOLUMNSID,OVERFLOWTEXT)
   VALUES (L_RQTLOGCOLUMNSID, L_OVERFLOWTEXT);
END @





/****** Object:  StoredProcedure RQINSERT_RQTLOGDOCUMENTS ******/
--    CREATE PROCEDURE RQINSERT_RQTLOGDOCUMENTS
--    (
--       L_RQTLOGID int,
--       L_RQDOCUMENTGUID varchar(40),
--       L_RQDOCUMENTMAPID varchar(40),
--       L_VERSIONNUMBER varchar(50),
--       L_RQDOCUMENT CLOB
--    )
--    DYNAMIC RESULT SETS 0
--    LANGUAGE SQL
--    MODIFIES SQL DATA
--    BEGIN
--    
--       -- insert column values into RQTLOG table
--       INSERT INTO RQTLOGDOCUMENTS (RQTLOGID,RQDOCUMENTGUID,RQDOCUMENTMAPID,VERSIONNUMBER,RQDOCUMENT)
--       VALUES (L_RQTLOGID, L_RQDOCUMENTGUID, L_RQDOCUMENTMAPID, L_VERSIONNUMBER, L_RQDOCUMENT);
--    END @





/****** Object:  StoredProcedure RQINS_TLOGCOLUMNS_FIELDVALUES ******/
CREATE PROCEDURE RQINS_TLOGCOLUMNS_FIELDVALUES
(
   L_RQTLOGID int,
   L_REQUIREMENTID int
)
DYNAMIC RESULT SETS 0
LANGUAGE SQL
MODIFIES SQL DATA
BEGIN
   -- insert column values into RQTLOGCOLUMNS table
   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r2.RQGUID,'REQUIREMENTID', ltrim(rtrim(varchar(char(bigint(r1.REQUIREMENTID))))),0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQUSERDEFINEDFIELDS r2 ON r1.FIELDID=r2.ID
   WHERE r1.REQUIREMENTID = L_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r3.RQGUID,'REQUIREMENTGUID',r2.RQGUID,0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQREQUIREMENTS r2 ON r1.REQUIREMENTID=r2.ID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r1.FIELDID=r3.ID
   WHERE r1.REQUIREMENTID = L_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r4.RQGUID,'REQUIREMENTMAPID',EXTERNALID,0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQREQUIREMENTS r2 ON r1.REQUIREMENTID=r2.ID
   INNER JOIN RQTLOGMAP r3 ON r2.RQGUID=r3.OBJECTGUID
   INNER JOIN RQUSERDEFINEDFIELDS r4 ON r1.FIELDID=r4.ID
   WHERE r1.REQUIREMENTID = L_REQUIREMENTID AND r3.OBJECTTYPE='RQREQUIREMENTS';

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r3.RQGUID,'FIELDVALUE',r4.FIELDVALUE,COALESCE(r4.OVERFLOWIND,0)
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r2.USERDEFINEDFIELDID=r3.ID
   LEFT OUTER JOIN RQUSERDEFINEDFIELDVALUES r4 ON r3.ID=r4.FIELDID AND r1.ID=r4.REQUIREMENTID
   WHERE r1.ID=L_REQUIREMENTID AND r3.DATATYPE<>0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r2.RQGUID,'OVERFLOWIND',ltrim(rtrim(varchar(char(bigint(r1.OVERFLOWIND))))),0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQUSERDEFINEDFIELDS r2 ON r1.FIELDID=r2.ID
   WHERE r1.REQUIREMENTID = L_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r2.RQGUID,'NUMERICVALUE',ltrim(rtrim(varchar(char(bigint(r1.NUMERICVALUE))))),0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQUSERDEFINEDFIELDS r2 ON r1.FIELDID=r2.ID
   WHERE r1.REQUIREMENTID = L_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r3.RQGUID,'FIELDGUID',r3.RQGUID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r2.USERDEFINEDFIELDID=r3.ID
   WHERE r1.ID=L_REQUIREMENTID AND r3.DATATYPE<>0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r3.RQGUID,'FIELDMAPID',r4.EXTERNALID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r2.USERDEFINEDFIELDID=r3.ID
   INNER JOIN RQTLOGMAP r4 ON r3.RQGUID=r4.OBJECTGUID
   WHERE r1.ID = L_REQUIREMENTID 
   AND r3.DATATYPE<>0 
   AND r4.OBJECTTYPE = 'RQUSERDEFINEDFIELDS';

   --insert column values into RQTLOGOVERFLOW table
   INSERT INTO RQTLOGOVERFLOW (RQTLOGCOLUMNSID,OVERFLOWTEXT)
   SELECT r1.ID, r2.OVERFLOWFIELDVALUE
   FROM RQTLOGCOLUMNS r1
   INNER JOIN RQOVERFLOWVALUES r2 ON ltrim(rtrim(r1.TID))=ltrim(rtrim(varchar(char(bigint(r2.OVERFLOWFIELDID)))))
   WHERE r1.RQTLOGID=L_RQTLOGID
   AND r1.TABLENAME='RQUSERDEFINEDFIELDVALUES'
   AND r1.COLUMNNAME='FIELDVALUE'
   AND r1.ISOVERFLOW=1
   AND r2.OVERFLOWREQUIREMENTID=L_REQUIREMENTID;
END  @





/****** Object:  StoredProcedure RQINS_TLOGCOLUMNS_LISTVALUES ******/
CREATE PROCEDURE RQINS_TLOGCOLUMNS_LISTVALUES
(
   L_RQTLOGID int,
   L_REQUIREMENTID int
)
DYNAMIC RESULT SETS 0
LANGUAGE SQL
MODIFIES SQL DATA
BEGIN

   -- insert column values into RQTLOGCOLUMNS table
   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r4.RQGUID,'REQUIREMENTGUID',r2.RQGUID,0
   FROM RQUSERDEFINEDLISTVALUES r1
   INNER JOIN RQREQUIREMENTS r2 ON r1.REQUIREMENTID=r2.ID
   INNER JOIN RQUSERDEFINEDLISTITEMS r3 ON r1.LISTITEMID=r3.ITEMID
   INNER JOIN RQUSERDEFINEDFIELDS r4 ON r1.FIELDID=r4.ID
   WHERE r1.REQUIREMENTID=L_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r5.RQGUID,'FIELDGUID',r5.RQGUID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   LEFT OUTER JOIN RQUSERDEFINEDLISTVALUES r3 ON r2.USERDEFINEDFIELDID=r3.FIELDID AND r1.ID=r3.REQUIREMENTID
   FULL OUTER JOIN RQUSERDEFINEDLISTITEMS r4 ON r3.LISTITEMID=r4.ITEMID
   INNER JOIN RQUSERDEFINEDFIELDS r5 ON r2.USERDEFINEDFIELDID=r5.ID
   WHERE r1.ID=L_REQUIREMENTID AND r5.DATATYPE=0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r5.RQGUID,'ITEMGUID',r4.RQGUID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   LEFT OUTER JOIN RQUSERDEFINEDLISTVALUES r3 ON r2.USERDEFINEDFIELDID=r3.FIELDID AND r1.ID=r3.REQUIREMENTID
   FULL OUTER JOIN RQUSERDEFINEDLISTITEMS r4 ON r3.LISTITEMID=r4.ITEMID
   INNER JOIN RQUSERDEFINEDFIELDS r5 ON r2.USERDEFINEDFIELDID=r5.ID
   WHERE r1.ID=L_REQUIREMENTID AND r5.DATATYPE=0;

   -- insert external MAP ID values
   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r5.RQGUID,'REQUIREMENTMAPID',r3.EXTERNALID,0
   FROM RQUSERDEFINEDLISTVALUES r1
   INNER JOIN RQREQUIREMENTS r2 ON r1.REQUIREMENTID=r2.ID
   INNER JOIN RQTLOGMAP r3 ON r2.RQGUID=r3.OBJECTGUID
   INNER JOIN RQUSERDEFINEDLISTITEMS r4 ON r1.LISTITEMID=r4.ITEMID
   INNER JOIN RQUSERDEFINEDFIELDS r5 ON r1.FIELDID=r5.ID
   WHERE r1.REQUIREMENTID=L_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r3.RQGUID,'FIELDMAPID',r4.EXTERNALID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r2.USERDEFINEDFIELDID=r3.ID
   INNER JOIN RQTLOGMAP r4 ON r3.RQGUID=r4.OBJECTGUID
   WHERE r1.ID=L_REQUIREMENTID AND r3.DATATYPE=0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT L_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r4.RQGUID,'ITEMMAPID',r3.EXTERNALID,0
   FROM RQUSERDEFINEDLISTVALUES r1
   INNER JOIN RQUSERDEFINEDLISTITEMS r2 ON r1.LISTITEMID=r2.ITEMID
   INNER JOIN RQTLOGMAP r3 ON r2.RQGUID=r3.OBJECTGUID
   INNER JOIN RQUSERDEFINEDFIELDS r4 ON r1.FIELDID=r4.ID
   WHERE r1.REQUIREMENTID=L_REQUIREMENTID;
END @







-- DROP PROCEDURE RQINS_TLOGCOLUMNS_REQ;
CREATE PROCEDURE RQINS_TLOGCOLUMNS_REQ
(
   TLOGID DECIMAL(10),
   REQUIREMENTID DECIMAL(10)
)
-- DYNAMIC RESULT SETS 0
LANGUAGE SQL
MODIFIES SQL DATA
BEGIN

   -- Declarations of RQRequirement columns values
   DECLARE L_REQUIREMENTID DECIMAL(10);
   DECLARE L_REQUIREMENTTYPEID DECIMAL(10);
   DECLARE L_TAG DECIMAL(10);
   DECLARE L_LASTCHILDTAG DECIMAL(10);
   DECLARE L_BOOKMARKNAME varchar(64);
   DECLARE L_BOOKMARKDOCUMENTID DECIMAL(10);
   DECLARE L_DOCUMENTPOSITION smallint;
   DECLARE L_LONGTAGNUMBER varchar(255);
   DECLARE L_REQUIREMENTPREFIX varchar(64);
   DECLARE L_REQUIREMENTTEXT varchar(4000);
   DECLARE L_REQOVERFLOWIND smallint;
   DECLARE L_DELETED smallint;
   DECLARE L_RQGUID varchar(40);
   DECLARE L_REQFLAGS DECIMAL(10);
   DECLARE L_LASTDOCTEXT DECIMAL(10);
   DECLARE L_REQUIREMENTNAME varchar(255);
   DECLARE L_SUSPECTDATETIME timestamp;

   -- Local variable declarations
   DECLARE L_LOGSCOLUMNTID DECIMAL(10);
   DECLARE L_LOGSOVERFLOWTEXT varchar(32672);
   DECLARE L_RQDOCGUID varchar(40);
   DECLARE L_RQDOCMAPID varchar(40);
   DECLARE L_RQREQTYPEGUID varchar(40);
   DECLARE L_RQREQTYPEMAPID varchar(40);
   DECLARE L_EXTERNALID varchar(40);
   
   -- output value
   DECLARE OUTV DECIMAL(10);
   
   SET OUTV=0;
   
   -- Populate values from the RQREQUIREMENTS table
   SELECT ID,
      REQUIREMENTTYPEID,
      TAG,
      LASTCHILDTAG,
      BOOKMARKNAME,
      BOOKMARKDOCUMENTID,
      DOCUMENTPOSITION,
      LONGTAGNUMBER,
      REQUIREMENTPREFIX,
      REQUIREMENTTEXT,
      OVERFLOWIND,
      DELETED,
      RQGUID,
      FLAGS,
      LASTDOCTEXT,
      REQUIREMENTNAME,
      SUSPECTDATETIME
   INTO 
      L_REQUIREMENTID,
      L_REQUIREMENTTYPEID,
      L_TAG,
      L_LASTCHILDTAG,
      L_BOOKMARKNAME,
      L_BOOKMARKDOCUMENTID,
      L_DOCUMENTPOSITION,
      L_LONGTAGNUMBER,
      L_REQUIREMENTPREFIX,
      L_REQUIREMENTTEXT,
      L_REQOVERFLOWIND,
      L_DELETED,
      L_RQGUID,
      L_REQFLAGS,
      L_LASTDOCTEXT,
      L_REQUIREMENTNAME,
      L_SUSPECTDATETIME
   FROM RQREQUIREMENTS
   WHERE ID = REQUIREMENTID;
   
   if L_REQUIREMENTID is null then
      SET L_REQUIREMENTID = 0;
   end if;
   
   if L_TAG is null then
      SET L_TAG = -1;
   end if;
   
   if L_LASTCHILDTAG is null then
      SET L_LASTCHILDTAG = -1;
   end if;
   
   if L_BOOKMARKNAME is null then
      SET L_BOOKMARKNAME = '';
   end if;
   
   if L_BOOKMARKDOCUMENTID is null then
      SET L_BOOKMARKDOCUMENTID = -1;
   end if;
   
   if L_DOCUMENTPOSITION is null then
      SET L_DOCUMENTPOSITION = -1;
   end if;
   
   if L_LONGTAGNUMBER is null then
      SET L_LONGTAGNUMBER = '';
   end if;

   if L_REQUIREMENTPREFIX is null then
      SET L_REQUIREMENTPREFIX = '';
   end if;

   if L_REQUIREMENTTEXT is null then
      SET L_REQUIREMENTTEXT = '';
   end if;

   if L_REQOVERFLOWIND is null then
      SET L_REQOVERFLOWIND = 0;
   end if;

   if L_DELETED is null then
      SET L_DELETED = -1;
   end if;

   if L_RQGUID is null then
      SET L_RQGUID = '';
   end if;

   if L_REQFLAGS is null then
      SET L_REQFLAGS = -1;
   end if;

   if L_LASTDOCTEXT is null then
      SET L_LASTDOCTEXT = -1;
   end if;

   if L_REQUIREMENTNAME is null then
      SET L_REQUIREMENTNAME = '';
   end if;

   if L_SUSPECTDATETIME is null then
      SET L_SUSPECTDATETIME = '';
   end if;

   IF (L_REQUIREMENTID IS NOT NULL) THEN
      -- get guids from id
      SELECT RQGUID INTO L_RQREQTYPEGUID FROM RQREQUIREMENTTYPES WHERE ID=L_REQUIREMENTTYPEID;
      SELECT RQGUID INTO L_RQDOCGUID FROM RQDOCUMENTS WHERE ID=L_BOOKMARKDOCUMENTID;

      -- get external map ids from RQTLOGMAP table
      SELECT EXTERNALID INTO L_RQREQTYPEMAPID FROM RQTLOGMAP WHERE OBJECTGUID=L_RQREQTYPEGUID AND OBJECTTYPE='RQREQUIREMENTTYPES';
      SELECT EXTERNALID INTO L_EXTERNALID FROM RQTLOGMAP WHERE OBJECTGUID=L_RQGUID AND OBJECTTYPE='RQREQUIREMENTS';
      SELECT EXTERNALID INTO L_RQDOCMAPID FROM RQTLOGMAP WHERE OBJECTGUID=L_RQDOCGUID AND OBJECTTYPE='RQDOCUMENTS';

      -- insert column values into RQTLOGCOLUMNS table
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'REQUIREMENTID',          ltrim(rtrim(varchar(char(bigint(L_REQUIREMENTID))))),      0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'REQUIREMENTTYPEGUID',    L_RQREQTYPEGUID,      0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'REQUIREMENTTYPEMAPID',   L_RQREQTYPEMAPID,     0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'TAG',                    ltrim(rtrim(varchar(char(bigint(L_TAG))))),                0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'LASTCHILDTAG',           ltrim(rtrim(varchar(char(bigint(L_LASTCHILDTAG))))),       0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'BOOKMARKNAME',           L_BOOKMARKNAME,       0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'BOOKMARKDOCUMENTID',     ltrim(rtrim(varchar(char(bigint(L_BOOKMARKDOCUMENTID))))), 0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'BOOKMARKDOCUMENTGUID',   L_RQDOCGUID,          0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'BOOKMARKDOCUMENTMAPID',  L_RQDOCMAPID,         0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'DOCUMENTPOSITION',       ltrim(rtrim(varchar(char(bigint(L_DOCUMENTPOSITION))))),   0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'LONGTAGNUMBER',          L_LONGTAGNUMBER,      0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'REQUIREMENTPREFIX',      L_REQUIREMENTPREFIX,  0, OUTV);

      -- Check whether an overflow exists. If exists populate the RQTLOGOVERFLOW table for the REQUIREMENTTEXT column entry
      IF L_REQOVERFLOWIND=1 THEN
         CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'REQUIREMENTTEXT', L_REQUIREMENTTEXT, 1, L_LOGSCOLUMNTID);
         SET L_LOGSOVERFLOWTEXT=(SELECT OVERFLOWTEXT FROM RQOVERFLOWTEXT WHERE OVERFLOWID=L_REQUIREMENTID);
         CALL RQINSERT_RQTLOGOVERFLOW (L_LOGSCOLUMNTID, L_LOGSOVERFLOWTEXT);
      ELSE
         CALL RQINSERT_RQTLOGCOLUMNS ( TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'REQUIREMENTTEXT', L_REQUIREMENTTEXT, 0, OUTV);
      END IF;

      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'OVERFLOWIND',     ltrim(rtrim(varchar(char(bigint(L_REQOVERFLOWIND))))),  0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'DELETED',         ltrim(rtrim(varchar(char(bigint(L_DELETED))))),         0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'RQGUID',          L_RQGUID,          0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'MAPID',           L_EXTERNALID,      0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'FLAGS',           ltrim(rtrim(varchar(char(bigint(L_REQFLAGS))))),        0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'LASTDOCTEXT',     ltrim(rtrim(varchar(char(bigint(L_LASTDOCTEXT))))),     0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'REQUIREMENTNAME', L_REQUIREMENTNAME, 0, OUTV);
      CALL RQINSERT_RQTLOGCOLUMNS (TLOGID, 'RQREQUIREMENTS', L_RQGUID, 'SUSPECTDATETIME', L_SUSPECTDATETIME, 0, OUTV);
   END IF;
END  @




CREATE PROCEDURE RQDELETE_RQTLOGMAP_ENTRY (L_OBJECTTYPE varchar(40), L_OBJECTGUID varchar(40))
BEGIN
   DECLARE L_RQREQUIREMENTID int;

   -- Delete the RQTLOGMAP entry for this requirement.
   DELETE FROM RQTLOGMAP WHERE OBJECTTYPE=L_OBJECTTYPE and OBJECTGUID=L_OBJECTGUID;

   IF (UPPER(RTRIM(L_OBJECTTYPE))='RQREQUIREMENTS') THEN
      -- Delete the RQRESOURCES entry for this requirement.
      SELECT ID INTO L_RQREQUIREMENTID FROM RQREQUIREMENTS WHERE RQGUID = L_OBJECTGUID;
      DELETE FROM RQRESOURCES WHERE FLAGS=2 AND REQUIREMENTID=L_RQREQUIREMENTID;
   END IF;
END  @



/****** Object:  StoredProcedure RQINSERT_LOG_REQUIREMENT ******/
CREATE PROCEDURE RQINSERT_LOG_REQUIREMENT
(
   PROJECTGUID varchar(40)
   ,REQUIREMENTGUID varchar(40)
   ,TRANSTYPE int
)
DYNAMIC RESULT SETS 0
LANGUAGE SQL
MODIFIES SQL DATA
BEGIN

   --Declarations of RQRequirement columns values
   DECLARE L_RQREQUIREMENTID int;
   DECLARE L_VERSIONUSERID int;
   DECLARE L_VERSIONDATETIME timestamp;
   DECLARE L_BOOKMARKDOCUMENTID int;

   --Local variable declarations
   DECLARE L_LOGSTID int;
   DECLARE L_ISDOC smallint;
   DECLARE L_DOCVERNUMBER varchar(50);
   DECLARE L_RQPROJECTMAPID varchar(40);
   DECLARE L_RQUSERID varchar(20);
   DECLARE L_RQDOCGUID varchar(40);
   DECLARE L_RQDOCMAPID varchar(40);
   DECLARE L_EXTERNALID varchar(40);
   
   SET L_LOGSTID=0;

   --Populate values from the RQREQUIREMENTS table
   SELECT 
      ID,
      BOOKMARKDOCUMENTID,
      VERSIONUSERID,
      VERSIONDATETIME
   INTO
       L_RQREQUIREMENTID, L_BOOKMARKDOCUMENTID, L_VERSIONUSERID, L_VERSIONDATETIME
   FROM RQREQUIREMENTS WHERE RQGUID = REQUIREMENTGUID;
   
   if L_BOOKMARKDOCUMENTID is null then
      SET L_BOOKMARKDOCUMENTID = -1;
   end if;

   if L_VERSIONUSERID is null then
      SET L_VERSIONUSERID = -1;
   end if;

   if L_VERSIONDATETIME is null then
      SET L_VERSIONDATETIME = (SELECT current timestamp FROM sysibm.sysdummy1);
   end if;


   If L_RQREQUIREMENTID IS NOT NULL THEN
      --Set default values
      SET L_ISDOC = 0;
      If L_BOOKMARKDOCUMENTID>0 THEN
         SET L_ISDOC = 1;
      END IF;

      -- get user guid and login name from id
      SELECT LOGINNAME INTO L_RQUSERID FROM RQUSERS WHERE ID=L_VERSIONUSERID;
      if L_RQUSERID is null then
         SET L_RQUSERID = 'UNKNOWN';
      end if;


      -- get external map ids from RQTLOGMAP table
      SELECT EXTERNALID INTO L_RQPROJECTMAPID FROM RQTLOGMAP WHERE OBJECTGUID=PROJECTGUID AND OBJECTTYPE='RQPROJECTS';
      SELECT EXTERNALID into L_EXTERNALID FROM RQTLOGMAP WHERE OBJECTGUID=REQUIREMENTGUID AND OBJECTTYPE='RQREQUIREMENTS';

      -- insert column values into RQTLOG table
      CALL RQINSERT_RQTLOG (PROJECTGUID,L_RQPROJECTMAPID,L_RQUSERID,L_VERSIONDATETIME,L_ISDOC,TRANSTYPE,6,REQUIREMENTGUID,L_EXTERNALID,L_LOGSTID);

      If TRANSTYPE <> 3 THEN
         --insert requirement source document to RQTLOGDOCUMENTS
--             If L_ISDOC=1 THEN
--                SELECT VERSIONNUMBER, RQGUID 
--                into L_DOCVERNUMBER, L_RQDOCGUID
--                FROM RQDOCUMENTS WHERE ID=L_BOOKMARKDOCUMENTID;
--    
--                SELECT EXTERNALID into L_RQDOCMAPID FROM RQTLOGMAP WHERE OBJECTGUID=L_RQDOCGUID AND OBJECTTYPE='RQDOCUMENTS';
--    
--                CALL RQINSERT_RQTLOGDOCUMENTS (L_LOGSTID, L_RQDOCGUID, L_RQDOCMAPID, L_DOCVERNUMBER, 'DOCUMENT');
--             END IF;

         --insert for RQREQUIREMENTS
         --insert column values into RQTLOGCOLUMNS table
         CALL RQINS_TLOGCOLUMNS_REQ (L_LOGSTID, L_RQREQUIREMENTID);

         --insert for RQUSERDEFINEDFIELDVALUES
         --insert column values into RQTLOGCOLUMNS table
         CALL RQINS_TLOGCOLUMNS_FIELDVALUES (L_LOGSTID, L_RQREQUIREMENTID);

         --insert for RQUSERDEFINEDLISTITEMS
         --insert column values into RQTLOGCOLUMNS table
         CALL RQINS_TLOGCOLUMNS_LISTVALUES (L_LOGSTID, L_RQREQUIREMENTID);
      END IF;
   END IF;
END  @




CREATE PROCEDURE RQDELETE_LOG_REQUIREMENT
(
   L_PROJECTGUID      varchar(40),
   L_REQUIREMENTGUID  varchar(40),
   L_VERSIONUSERID    int
)
BEGIN
   --Local variable declarations
   DECLARE L_LOGSTID int;
   DECLARE L_RQPROJECTMAPID varchar(40);
   DECLARE L_RQUSERID varchar(20);
   DECLARE L_EXTERNALID varchar(40);
   DECLARE L_VERSIONDATETIME  TIMESTAMP;
   DECLARE RQPROJECTFLAGS          int;
   
   SET RQPROJECTFLAGS = 0;
   SELECT FLAGS INTO RQPROJECTFLAGS FROM RQPROJECTS WHERE RQGUID=L_PROJECTGUID;
   
   IF ((UDFBITAND(RQPROJECTFLAGS, 64)) <> 0) THEN -- Fisrt check sync flag 
      IF (  (SELECT COUNT(*) FROM RQTLOGMAP WHERE OBJECTGUID=L_REQUIREMENTGUID AND OBJECTTYPE='RQREQUIREMENTS' AND EXTERNALID IS NOT NULL) > 0 ) THEN

         SELECT current timestamp INTO L_VERSIONDATETIME FROM sysibm.sysdummy1;

         -- get user guid and login name from id
         SELECT LOGINNAME INTO L_RQUSERID FROM RQUSERS WHERE ID=L_VERSIONUSERID;
         if L_RQUSERID is null then
            SET L_RQUSERID = 'UNKNOWN';
         end if;

         -- get external map ids from RQTLOGMAP table
         SELECT EXTERNALID INTO L_RQPROJECTMAPID FROM RQTLOGMAP WHERE OBJECTGUID=L_PROJECTGUID     AND OBJECTTYPE='RQPROJECTS';
         SELECT EXTERNALID INTO L_EXTERNALID     FROM RQTLOGMAP WHERE OBJECTGUID=L_REQUIREMENTGUID AND OBJECTTYPE='RQREQUIREMENTS';

         -- insert column values into RQTLOG table
         CALL RQINSERT_RQTLOG (L_PROJECTGUID, L_RQPROJECTMAPID, L_RQUSERID, L_VERSIONDATETIME, 0, 3, 6, L_REQUIREMENTGUID, L_EXTERNALID, L_LOGSTID);

         -- Delete the requirement sync information for this requirement
         CALL RQDELETE_RQTLOGMAP_ENTRY ('RQREQUIREMENTS', L_REQUIREMENTGUID);

      END IF;
   END IF;
END @



--    /****** Object:  StoredProcedure MIGRATE_TLOGMAP ******/
--    CREATE PROCEDURE MIGRATE_TLOGMAP
--    (
--       IN PROJECTGUID varchar(40),
--       IN LOGINNAME varchar(20),
--       OUT MSTATUS int
--    )
--    DYNAMIC RESULT SETS 0
--    LANGUAGE SQL
--    MODIFIES SQL DATA
--    BEGIN
--    
--       --Local variable declarations
--       DECLARE RQPROJECTFLAGS INT;
--       DECLARE ERROR INT;
--       DECLARE p_reqid INT;
--       DECLARE inputV BLOB;
--       DECLARE RQGUID VARCHAR(40);
--       DECLARE RRCID VARCHAR(40);
--    
--       SET MSTATUS = 0; -- initialize return value
--       SET ERROR=0;
--    
--       -- Validate that he user specified in RQTLOGPROPERTY table is real and that it is part of the project referenced by the RQGUID.
--       IF ( (select count(*) from rqprojects p, rqprojectusers pu, RQUSERS u, RQUSERGROUPS ug where p.id = pu.PROJECTID and pu.USERGROUPID = u.GROUPID and p.RQGUID=PROJECTGUID and u.loginname=LOGINNAME and pu.usergroupid = ug.id) > 0 ) THEN
--    
--          -- get project guid, name and flags from id
--          SELECT FLAGS INTO RQPROJECTFLAGS FROM RQPROJECTS WHERE RQGUID=PROJECTGUID;
--    
--          -- Check if managed requirements and sync project.
--          -- if true, write to TLOG tables
--          If (( BITAND(RQPROJECTFLAGS, 64)) = 0) THEN
--             -- populate values into RQTLOGMAP
         --    
--             SELECT
--                r1.ID,
--                r1.RQGUID,
--                r2.RESOURCEOBJ
--             INTO
--                p_reqid,
--                RQGUID,
--                inputV
--             FROM
--                RQREQUIREMENTS r1
--                INNER JOIN RQRESOURCES r2 ON r1.ID=r2.REQUIREMENTID
--                LEFT OUTER JOIN RQTLOGMAP r3 ON r1.RQGUID=r3.OBJECTGUID
--                INNER JOIN RQPROJECTREQUIREMENTS r4 ON r1.ID=r4.REQUIREMENTID
--                INNER JOIN RQPROJECTS r5 ON r4.PROJECTID=r5.ID
--             WHERE
--                (BITAND(r2.FLAGS, 2))<>0 AND
--                r3.EXTERNALID IS NULL AND
--                r5.RQGUID=PROJECTGUID;
            --    
--             -- Extract the composer id for this requirement
--             CALL RqGetRRCID(p_reqid, RRCID);
--    
--             -- insert values into RQTLOGMAP
--             INSERT INTO RQTLOGMAP(OBJECTTYPE, OBJECTGUID, EXTERNALID) values ('RQREQUIREMENTS', RQGUID, RRCID);
--    
--             -- update project FLAGS and RRMSYNCUSERID values
--             UPDATE RQPROJECTS SET FLAGS=BITOR(FLAGS, 64) WHERE RQGUID=PROJECTGUID;
--             INSERT INTO RQTLOGPROPERTY VALUES (PROJECTGUID, 'RRMSYNCUSERID', LOGINNAME );
--          END IF;
--    
--          If (ERROR <> 0) THEN
--             SET MSTATUS = 0;
--             ROLLBACK;
--          ELSE
--             SET MSTATUS = 1;
--             COMMIT;
--          END IF;
--       END IF;
--    END @









CREATE VIEW RQTLOGV_SYNCUSR AS
      SELECT
         RQPROJECTS.FLAGS,
         RQPROJECTS.RQGUID RQPROJECTS_RQGUID,
         RQREQUIREMENTS.RQGUID RQREQUIREMENTS_RQGUID,
         RQTLOGPROPERTY.VALUE,
         RQTLOGPROPERTY.KEYNAME,
         RQPROJECTREQUIREMENTS.REQUIREMENTID
      FROM
         RQPROJECTS INNER JOIN
         RQPROJECTREQUIREMENTS ON RQPROJECTS.ID = RQPROJECTREQUIREMENTS.PROJECTID INNER JOIN
         RQTLOGPROPERTY ON RQPROJECTS.RQGUID = RQTLOGPROPERTY.PROJECTGUID INNER JOIN
         RQREQUIREMENTS ON RQPROJECTREQUIREMENTS.REQUIREMENTID = RQREQUIREMENTS.ID@
         
         




CREATE TRIGGER RQREQHIST_TRIG_LOG AFTER INSERT ON RQREQUIREMENTHISTORY
REFERENCING NEW AS N
FOR EACH ROW MODE DB2SQL
BEGIN ATOMIC

    -- Declarations of the current columns values
   DECLARE RQREQUIREMENTHISTORYID  DECIMAL (10);
   DECLARE VERSIONUSERID           DECIMAL (10);
   DECLARE VERSIONNUMBER           varchar (50);

   --Declarations of RQRequirement columns values
   DECLARE RQREQGUID               varchar(40);

   --Local variable declarations
   DECLARE RQPROJECTGUID           varchar(40);
   DECLARE VERSIONUSERNAME         varchar(40);
   DECLARE RQPROJECTFLAGS          int;
   DECLARE RRMSYNCUSERNAME         varchar(40);
   
   SET RQREQUIREMENTHISTORYID=0;
   SET VERSIONUSERID=0;
   SET VERSIONNUMBER='1.0001';
   SET RQREQGUID = '';
   
   SET RQPROJECTGUID = '';
   SET VERSIONUSERNAME = '';
   SET RQPROJECTFLAGS = 0;
   SET RRMSYNCUSERNAME = '';
   
   -- Get values from the base table
   
   SET RQREQUIREMENTHISTORYID = N.ID;
   SET VERSIONUSERID =          N.VERSIONUSERID;
   SET VERSIONNUMBER =          N.VERSIONNUMBER;

   -- Do not start work unless this is an update to an existing requirement.
   IF ( VERSIONNUMBER <> '1.0000') THEN
      -- Get login name of the user who updated the RequisitePro requirement.
      SET VERSIONUSERNAME = (SELECT LOGINNAME FROM RqUsers WHERE id=VERSIONUSERID);
      
      -- get project guid and flags from RQPROJECTS, userid of the updater, userid from RqUsers
      -- All from a single query to take advantage of the schema relationship and save calls and time.
      SET RQPROJECTFLAGS =  (SELECT FLAGS                 FROM RQTLOGV_SYNCUSR WHERE (KEYNAME = 'RRMSyncUser') AND (REQUIREMENTID = RQREQUIREMENTHISTORYID));
      SET RQPROJECTGUID =   (SELECT RQPROJECTS_RQGUID     FROM RQTLOGV_SYNCUSR WHERE (KEYNAME = 'RRMSyncUser') AND (REQUIREMENTID = RQREQUIREMENTHISTORYID));
      SET RQREQGUID =       (SELECT RQREQUIREMENTS_RQGUID FROM RQTLOGV_SYNCUSR WHERE (KEYNAME = 'RRMSyncUser') AND (REQUIREMENTID = RQREQUIREMENTHISTORYID));
      SET RRMSYNCUSERNAME = (SELECT VALUE                 FROM RQTLOGV_SYNCUSR WHERE (KEYNAME = 'RRMSyncUser') AND (REQUIREMENTID = RQREQUIREMENTHISTORYID));
      
      IF ( ((UDFBITAND(RQPROJECTFLAGS, 64)) <> 0) AND (RRMSYNCUSERNAME <> VERSIONUSERNAME) ) THEN
         IF (SELECT COUNT(*) FROM RQTLOGMAP WHERE OBJECTGUID=RQREQGUID AND OBJECTTYPE='RQREQUIREMENTS' AND EXTERNALID IS NOT NULL) > 0 THEN
            -- insert entries into TLog tables for Requirement
            CALL RQINSERT_LOG_REQUIREMENT (RQPROJECTGUID,RQREQGUID,2);
         END IF;
      END IF;
   END IF;
END @


-- CREATE TRIGGER RQTLOGMAP_DTRIG AFTER DELETE ON RQTLOGMAP
-- REFERENCING OLD AS DELETED
-- FOR EACH ROW
-- WHEN (DELETED.OBJECTTYPE='RQPROJECTS')
-- BEGIN ATOMIC
-- 
--   IF DELETED.OBJECTTYPE='RQPROJECTS' THEN
-- 
--       -- Remove integration data for this project
--       DELETE FROM RQTLOGPROPERTY WHERE PROJECTGUID=DELETED.OBJECTGUID;
-- 
--       -- Remove entries in RqTLogMap table related to this project
-- 
--       -- DELETE THE RQTLOGMAP RECORDS FOR RQREQUIREMENTS
--       DELETE FROM RQTLOGMAP WHERE ID IN (
--          SELECT
--             M.ID
--          FROM
--             RQTLOGMAP                  M,
--             RQPROJECTS                 P,
--             RQPROJECTREQUIREMENTS      PR,
--             RQREQUIREMENTS             R
--          WHERE
--             P.ID=PR.PROJECTID          AND
--             PR.REQUIREMENTID= R.ID     AND
--             R.RQGUID=M.OBJECTGUID      AND
--             P.RQGUID=DELETED.OBJECTGUID
--       ) AND OBJECTTYPE='RQREQUIREMENTS';
-- 
--       -- DELETE THE RQTLOGMAP RECORDS FOR RQREQUIREMENTTYPES
--       DELETE FROM RQTLOGMAP WHERE ID IN (
--          SELECT
--             M.ID
--          FROM
--             RQTLOGMAP                  M,
--             RQPROJECTS                 P,
--             RQPROJECTREQUIREMENTTYPES  PT,
--             RQREQUIREMENTTYPES         T
--          WHERE
--             P.ID=PT.PROJECTID          AND
--             PT.REQUIREMENTTYPEID= T.ID AND
--             T.RQGUID=M.OBJECTGUID      AND 
--             P.RQGUID=DELETED.OBJECTGUID
--       ) AND OBJECTTYPE='RQREQUIREMENTTYPES';
-- 
--       -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDFIELDS
--       DELETE FROM RQTLOGMAP WHERE ID IN (
--          SELECT
--             M.ID
--          FROM
--             RQTLOGMAP                  M,
--             RQPROJECTS                 P,
--             RQPROJECTREQUIREMENTTYPES  PT,
--             RQREQUIREMENTTYPES         T,
--             RQREQUIREMENTTYPEFIELDS    TF,
--             RQUSERDEFINEDFIELDS        F
--          WHERE
--             P.ID=PT.PROJECTID          AND
--             PT.REQUIREMENTTYPEID= T.ID AND
--             T.ID=TF.REQUIREMENTTYPEID  AND
--             TF.USERDEFINEDFIELDID=F.ID AND
--             F.RQGUID=M.OBJECTGUID      AND 
--             P.RQGUID=DELETED.OBJECTGUID
--       ) AND OBJECTTYPE='RQUSERDEFINEDFIELDS';
-- 
--       -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDLISTITEMS
--       DELETE FROM RQTLOGMAP WHERE ID IN (
--          SELECT
--             M.ID
--          FROM
--             RQTLOGMAP                  M,
--             RQPROJECTS                 P,
--             RQPROJECTREQUIREMENTTYPES  PT,
--             RQREQUIREMENTTYPES         T,
--             RQREQUIREMENTTYPEFIELDS    TF,
--             RQUSERDEFINEDFIELDS        F,
--             RQUSERDEFINEDLISTITEMS     I
--          WHERE
--             P.ID=PT.PROJECTID          AND
--             PT.REQUIREMENTTYPEID= T.ID AND
--             T.ID=TF.REQUIREMENTTYPEID  AND
--             TF.USERDEFINEDFIELDID=F.ID AND
--             F.ID=I.FIELDID             AND
--             I.RQGUID=M.OBJECTGUID      AND 
--             P.RQGUID=DELETED.OBJECTGUID
--       ) AND OBJECTTYPE='RQUSERDEFINEDLISTITEMS';
-- 
--       -- Remove RRC link for this project's requirements
--       DELETE FROM RQRESOURCES WHERE ID IN ( 
--          SELECT 
--             RS.ID 
--          FROM 
--             RQREQUIREMENTS             R, 
--             RQPROJECTREQUIREMENTS      PR, 
--             RQPROJECTS                 P, 
--             RQRESOURCES                RS  
--          WHERE 
--             R.ID = PR.REQUIREMENTID    AND 
--             P.ID = PR.PROJECTID        AND 
--             R.ID = RS.REQUIREMENTID    AND 
--             RS.FLAGS=2                 AND 
--             P.RQGUID=DELETED.OBJECTGUID 
--       );
-- 
--       --  Turn sync flag OFF, if it is ON, for this project
--       UPDATE RQPROJECTS SET FLAGS=(FLAGS-64) WHERE RQGUID=DELETED.OBJECTGUID AND (UDFBITAND(Integer(FLAGS),64)=64);
-- 
-- --   END IF;
-- 
-- END @




-- DROP PROCEDURE RQDEL_RQTLOGMAP;
CREATE PROCEDURE RQDEL_RQTLOGMAP
(
   IN L_PRJGUID    varchar(40),
   IN L_OBJECTTYPE varchar(30),
   IN L_UNIQUEID   varchar(40),
   IN L_IDTYPE        INT
)
DYNAMIC RESULT SETS 0
LANGUAGE SQL
MODIFIES SQL DATA
BEGIN

   DECLARE L_TLOGMAPID int;
   DECLARE L_REQID int;
   DECLARE L_REQTYPEID int;
   DECLARE L_OBJECTGUID varchar(40);

   SET L_TLOGMAPID=0;
   SET L_REQID=0;
   SET L_REQTYPEID=0;
   SET L_OBJECTGUID='';


   IF (L_IDTYPE=0) THEN-- the unique id is an external id

      IF (L_OBJECTTYPE='RQREQUIREMENTS') THEN
         SELECT
            M.ID, 
            R.RQGUID 
         INTO 
            L_TLOGMAPID, 
            L_OBJECTGUID 
         FROM 
            RQTLOGMAP               M, 
            RQPROJECTS              P, 
            RQPROJECTREQUIREMENTS   PR, 
            RQREQUIREMENTS          R 
         WHERE 
            P.RQGUID=L_PRJGUID      AND 
            P.ID=PR.PROJECTID       AND 
            PR.REQUIREMENTID=R.ID   AND 
            R.RQGUID=M.OBJECTGUID   AND 
            M.EXTERNALID=L_UNIQUEID
         FETCH FIRST ROW ONLY;
      END IF;
      
      IF (L_OBJECTTYPE='RQPROJECTS') THEN
         SELECT ID INTO L_TLOGMAPID FROM RQTLOGMAP WHERE OBJECTTYPE=L_OBJECTTYPE AND OBJECTGUID=L_PRJGUID AND EXTERNALID=L_UNIQUEID FETCH FIRST ROW ONLY;
         SET L_OBJECTGUID=L_PRJGUID;
      END IF;
      
      IF (L_OBJECTTYPE='RQREQUIREMENTTYPES') THEN
         SELECT 
            M.ID,
            T.RQGUID
         INTO 
            L_TLOGMAPID, 
            L_OBJECTGUID 
         FROM 
            RQTLOGMAP                  M, 
            RQPROJECTS                 P, 
            RQPROJECTREQUIREMENTTYPES  PT, 
            RQREQUIREMENTTYPES         T
         WHERE 
            P.RQGUID=L_PRJGUID         AND
            P.ID=PT.PROJECTID          AND
            PT.REQUIREMENTTYPEID= T.ID AND
            T.RQGUID=M.OBJECTGUID      AND
            M.EXTERNALID=L_UNIQUEID;
      END IF;
      
      -- # Bill Betz 12/14/2009: L_IDTYPE=0 is invalid for attributes and list items because there is no way to ensure uniqueness even within the project.
      IF (L_OBJECTTYPE='RQUSERDEFINEDFIELDS' OR L_OBJECTTYPE='RQUSERDEFINEDLISTITEMS') THEN
         -- SELECT ID, OBJECTGUID INTO L_TLOGMAPID, L_OBJECTGUID FROM RQTLOGMAP WHERE OBJECTTYPE=L_OBJECTTYPE AND EXTERNALID=L_UNIQUEID FETCH FIRST ROW ONLY;
         SET L_TLOGMAPID  = 0;
         SET L_OBJECTGUID = '';
      END IF;

   ELSEIF   (L_IDTYPE=1) then-- the unique id is an RQGUID
      SET L_OBJECTGUID=L_UNIQUEID;
      SELECT ID             INTO L_TLOGMAPID               FROM RQTLOGMAP WHERE OBJECTTYPE=L_OBJECTTYPE AND OBJECTGUID=L_OBJECTGUID FETCH FIRST ROW ONLY;
   end if;

   if(L_TLOGMAPID<>0) then

      -- ##################################################################################################################
      -- ##################################################################################################################
      -- ##################################################################################################################
      IF (L_OBJECTTYPE='RQREQUIREMENTS') THEN

         -- First validate that the requirement belongs to the specified project.
         SELECT R.ID INTO L_REQID FROM RQREQUIREMENTS R, RQPROJECTREQUIREMENTS PR, RQPROJECTS P WHERE R.ID=PR.REQUIREMENTID AND PR.PROJECTID=P.ID AND R.RQGUID=L_OBJECTGUID AND P.RQGUID=L_PRJGUID;

         IF(L_REQID<>0) THEN

            -- DELETE THE RQTLOGMAP RECORDS FOR RQREQUIREMENTS
            DELETE FROM RQTLOGMAP WHERE ID=L_TLOGMAPID;

            -- Delete the RRC link in RQRESOURCES table (flags=2) for this requirement.
            -- Delete the RRC rich text in RQRESOURCES table (flags=1) for this requirement.
            DELETE FROM RQRESOURCES WHERE FLAGS IN (1,2) AND REQUIREMENTID=L_REQID;

            -- Set the non-text flag to OFF for this requirement, by turning off the 2nd bit in the respective RQREQUIREMENTS.FLAGS field.
            UPDATE RQREQUIREMENTS SET FLAGS=(FLAGS-2) WHERE RQGUID=L_OBJECTGUID AND (UDFBITAND(INTEGER(FLAGS),2)=2) AND ID=L_REQID;

         END IF;

      END IF;

      -- ##################################################################################################################
      -- ##################################################################################################################
      -- ##################################################################################################################
      IF (L_OBJECTTYPE='RQPROJECTS' AND L_OBJECTGUID=L_PRJGUID) THEN

         -- Delete all entries in RQTLOGPROPERTY for this project.
         DELETE FROM RQTLOGPROPERTY WHERE PROJECTGUID=L_OBJECTGUID;

         -- Delete the RRC link in RQRESOURCES table (flags=2) for this project.
         -- Delete the RRC rich text in RQRESOURCES table (flags=1) for this project.
         DELETE FROM RQRESOURCES WHERE ID IN (
            SELECT
               RS.ID
            FROM
               RQREQUIREMENTS             R,
               RQPROJECTREQUIREMENTS      PR,
               RQPROJECTS                 P,
               RQRESOURCES                RS,
               RQTLOGMAP                  M
            WHERE
               R.ID = PR.REQUIREMENTID    AND
               P.ID = PR.PROJECTID        AND
               R.ID = RS.REQUIREMENTID    AND
               RS.FLAGS IN(1,2)           AND
               M.OBJECTGUID=R.RQGUID      AND
               P.RQGUID=L_OBJECTGUID      AND
               M.OBJECTTYPE='RQREQUIREMENTS' 
         );
         
         -- Set the non-text flag to OFF for all requirements for this project, by turning off the 2nd bit in the respective RQREQUIREMENTS.FLAGS field.
         UPDATE RQREQUIREMENTS SET FLAGS=(FLAGS-2) WHERE (UDFBITAND(INTEGER(FLAGS),2)=2) AND ID IN (
            SELECT DISTINCT
               R.ID
            FROM
               RQREQUIREMENTS             R,
               RQPROJECTREQUIREMENTS      PR,
               RQPROJECTS                 P,
               RQTLOGMAP                  M
            WHERE
               R.ID = PR.REQUIREMENTID    AND
               P.ID = PR.PROJECTID        AND
               M.OBJECTGUID=R.RQGUID      AND
               P.RQGUID=L_OBJECTGUID      AND
               M.OBJECTTYPE='RQREQUIREMENTS' 
         );

         -- Remove entries in RqTLogMap table related to this project

         -- DELETE THE RQTLOGMAP RECORDS FOR RQREQUIREMENTTYPES
         DELETE FROM RQTLOGMAP WHERE ID IN (
            SELECT
               M.ID
            FROM
               RQTLOGMAP                  M,
               RQPROJECTS                 P,
               RQPROJECTREQUIREMENTTYPES  PT,
               RQREQUIREMENTTYPES         T
            WHERE
               P.ID=PT.PROJECTID          AND
               PT.REQUIREMENTTYPEID= T.ID AND
               T.RQGUID=M.OBJECTGUID      AND
               P.RQGUID=L_OBJECTGUID      AND 
               M.OBJECTTYPE='RQREQUIREMENTTYPES'
         );

         -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDFIELDS
         DELETE FROM RQTLOGMAP WHERE ID IN (
            SELECT
               M.ID
            FROM
               RQTLOGMAP                  M,
               RQPROJECTS                 P,
               RQPROJECTREQUIREMENTTYPES  PT,
               RQREQUIREMENTTYPES         T,
               RQREQUIREMENTTYPEFIELDS    TF,
               RQUSERDEFINEDFIELDS        F
            WHERE
               P.ID=PT.PROJECTID          AND
               PT.REQUIREMENTTYPEID= T.ID AND
               T.ID=TF.REQUIREMENTTYPEID  AND
               TF.USERDEFINEDFIELDID=F.ID AND
               F.RQGUID=M.OBJECTGUID      AND
               P.RQGUID=L_OBJECTGUID      AND
               M.OBJECTTYPE='RQUSERDEFINEDFIELDS'
         );

         -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDLISTITEMS
         DELETE FROM RQTLOGMAP WHERE ID IN (
            SELECT
               M.ID
            FROM
               RQTLOGMAP                  M,
               RQPROJECTS                 P,
               RQPROJECTREQUIREMENTTYPES  PT,
               RQREQUIREMENTTYPES         T,
               RQREQUIREMENTTYPEFIELDS    TF,
               RQUSERDEFINEDFIELDS        F,
               RQUSERDEFINEDLISTITEMS     I
            WHERE
               P.ID=PT.PROJECTID          AND
               PT.REQUIREMENTTYPEID= T.ID AND
               T.ID=TF.REQUIREMENTTYPEID  AND
               TF.USERDEFINEDFIELDID=F.ID AND
               F.ID=I.FIELDID             AND
               I.RQGUID=M.OBJECTGUID      AND
               P.RQGUID=L_OBJECTGUID      AND
               M.OBJECTTYPE='RQUSERDEFINEDLISTITEMS'
         );
         
         -- DELETE THE RQTLOGMAP RECORDS FOR RQREQUIREMENTS
         DELETE FROM RQTLOGMAP WHERE ID IN (
            SELECT
               M.ID
            FROM
               RQTLOGMAP                  M,
               RQPROJECTS                 P,
               RQPROJECTREQUIREMENTS      PR,
               RQREQUIREMENTS             R
            WHERE
               P.ID=PR.PROJECTID          AND
               PR.REQUIREMENTID= R.ID     AND
               R.RQGUID=M.OBJECTGUID      AND
               P.RQGUID=L_OBJECTGUID      AND 
               M.OBJECTTYPE='RQREQUIREMENTS'
         );

         --  Turn sync flag OFF, if it is ON, for this project
         UPDATE RQPROJECTS SET FLAGS=(FLAGS-64) WHERE RQGUID=L_OBJECTGUID AND (UDFBITAND(Integer(FLAGS),64)=64);

         -- Delete the entry in RQTLOGMAP for this project.
         DELETE FROM RQTLOGMAP WHERE OBJECTGUID=L_OBJECTGUID;

         -- Delete all current entries in RQTLOG for this project, related entries in RQTLOGCOLUMNS are deleted by cascade.
         DELETE FROM RQTLOG where PROJECTGUID=L_OBJECTGUID;

      END IF;

      -- ##################################################################################################################
      -- ##################################################################################################################
      -- ##################################################################################################################
      IF (L_OBJECTTYPE='RQREQUIREMENTTYPES') THEN

         -- First validate that the requirement belongs to the specified project.
         SELECT RT.ID INTO L_REQTYPEID FROM RQREQUIREMENTTYPES RT, RQPROJECTREQUIREMENTTYPES PRT, RQPROJECTS P WHERE RT.ID=PRT.REQUIREMENTTYPEID AND PRT.PROJECTID=P.ID AND P.RQGUID=L_PRJGUID AND RT.RQGUID=L_OBJECTGUID;

         IF (L_REQTYPEID<>0) THEN

            -- Delete the RRC link in RQRESOURCES table (flags=2) for this requirement.
            -- Delete the RRC rich text in RQRESOURCES table (flags=1) for this requirement.
            DELETE FROM RQRESOURCES WHERE FLAGS IN (1,2) AND ID IN (
               SELECT
                  RS.ID
               FROM
                  RQREQUIREMENTS             R,
                  RQPROJECTREQUIREMENTS      PR,
                  RQPROJECTS                 P,
                  RQRESOURCES                RS,
                  RQTLOGMAP                  M,
                  RQREQUIREMENTTYPES         RT 
               WHERE
                  R.ID = PR.REQUIREMENTID    AND
                  PR.PROJECTID = P.ID        AND
                  P.RQGUID=L_PRJGUID         AND
                  R.ID = RS.REQUIREMENTID    AND
                  RS.FLAGS IN(1,2)           AND
                  M.OBJECTGUID=R.RQGUID      AND
                  RT.ID=R.REQUIREMENTTYPEID  AND
                  RT.RQGUID=L_OBJECTGUID     AND
                  M.OBJECTTYPE='RQREQUIREMENTS'
            );

            -- Set the non-text flag to OFF for this requirement, by turning off the 2nd bit in the respective RQREQUIREMENTS.FLAGS field.
            UPDATE RQREQUIREMENTS SET FLAGS=(FLAGS-2) WHERE (UDFBITAND(INTEGER(FLAGS),2)=2) and ID IN 
            (
               SELECT R.ID 
               FROM 
                  RQREQUIREMENTS             R, 
                  RQREQUIREMENTTYPES         RT,
                  RQTLOGMAP                  M
               WHERE 
                  RT.ID=R.REQUIREMENTTYPEID  AND 
                  RT.RQGUID=L_OBJECTGUID     AND
                  M.OBJECTGUID=R.RQGUID      AND
                  M.OBJECTTYPE='RQREQUIREMENTS'
            );

            -- DELETE THE RQTLOGMAP RECORDS FOR RQREQUIREMENTTYPES
            DELETE FROM RQTLOGMAP WHERE ID IN (
               SELECT
                  M.ID
               FROM
                  RQTLOGMAP                  M,
                  RQPROJECTS                 P,
                  RQPROJECTREQUIREMENTTYPES  PT,
                  RQREQUIREMENTTYPES         T
               WHERE
                  P.ID=PT.PROJECTID          AND
                  PT.REQUIREMENTTYPEID= T.ID AND
                  T.RQGUID=M.OBJECTGUID      AND
                  M.OBJECTGUID=L_OBJECTGUID  AND
                  M.OBJECTTYPE='RQREQUIREMENTTYPES'
            );

            -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDFIELDS
            DELETE FROM RQTLOGMAP WHERE ID IN (
               SELECT
                  M.ID
               FROM
                  RQTLOGMAP                  M,
                  RQPROJECTS                 P,
                  RQPROJECTREQUIREMENTTYPES  PT,
                  RQREQUIREMENTTYPES         T,
                  RQREQUIREMENTTYPEFIELDS    TF,
                  RQUSERDEFINEDFIELDS        F
               WHERE
                  P.ID=PT.PROJECTID          AND
                  PT.REQUIREMENTTYPEID= T.ID AND
                  T.ID=TF.REQUIREMENTTYPEID  AND
                  TF.USERDEFINEDFIELDID=F.ID AND
                  F.RQGUID=M.OBJECTGUID      AND
                  T.RQGUID=L_OBJECTGUID      AND
                  M.OBJECTTYPE='RQUSERDEFINEDFIELDS'
            );

            -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDLISTITEMS
            DELETE FROM RQTLOGMAP WHERE ID IN (
               SELECT
                  M.ID
               FROM
                  RQTLOGMAP                  M,
                  RQPROJECTS                 P,
                  RQPROJECTREQUIREMENTTYPES  PT,
                  RQREQUIREMENTTYPES         T,
                  RQREQUIREMENTTYPEFIELDS    TF,
                  RQUSERDEFINEDFIELDS        F,
                  RQUSERDEFINEDLISTITEMS     I
               WHERE
                  P.ID=PT.PROJECTID          AND
                  PT.REQUIREMENTTYPEID= T.ID AND
                  T.ID=TF.REQUIREMENTTYPEID  AND
                  TF.USERDEFINEDFIELDID=F.ID AND
                  F.ID=I.FIELDID             AND
                  I.RQGUID=M.OBJECTGUID      AND
                  T.RQGUID=L_OBJECTGUID      AND
                  M.OBJECTTYPE='RQUSERDEFINEDLISTITEMS'
            );

            -- DELETE THE RQTLOGMAP RECORDS FOR RQREQUIREMENTS
            DELETE FROM RQTLOGMAP WHERE ID IN (
               SELECT
                  M.ID
               FROM
                  RQTLOGMAP                  M,
                  RQPROJECTS                 P,
                  RQPROJECTREQUIREMENTS      PR,
                  RQREQUIREMENTS             R,
                  RQREQUIREMENTTYPES         RT
               WHERE
                  P.ID=PR.PROJECTID          AND
                  PR.REQUIREMENTID= R.ID     AND
                  R.RQGUID=M.OBJECTGUID      AND
                  RT.ID=R.REQUIREMENTTYPEID  AND
                  RT.RQGUID=L_OBJECTGUID     AND
                  M.OBJECTTYPE='RQREQUIREMENTS'
            );

         END IF;

      END IF;

      -- ##################################################################################################################
      -- ##################################################################################################################
      -- ##################################################################################################################
      IF (L_OBJECTTYPE='RQUSERDEFINEDFIELDS') THEN
      
         -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDLISTITEMS
         DELETE FROM RQTLOGMAP WHERE ID IN (
            SELECT
               M.ID
            FROM
               RQTLOGMAP                  M,
               RQPROJECTS                 P,
               RQPROJECTREQUIREMENTTYPES  PT,
               RQREQUIREMENTTYPES         T,
               RQREQUIREMENTTYPEFIELDS    TF,
               RQUSERDEFINEDFIELDS        F,
               RQUSERDEFINEDLISTITEMS     I
            WHERE
               P.ID=PT.PROJECTID          AND
               PT.REQUIREMENTTYPEID= T.ID AND
               T.ID=TF.REQUIREMENTTYPEID  AND
               TF.USERDEFINEDFIELDID=F.ID AND
               F.ID=I.FIELDID             AND
               I.RQGUID=M.OBJECTGUID      AND
               F.RQGUID=L_OBJECTGUID
         );

         -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDFIELDS
         DELETE FROM RQTLOGMAP WHERE ID=L_TLOGMAPID;

      END IF;

      -- ##################################################################################################################
      -- ##################################################################################################################
      -- ##################################################################################################################
      IF (L_OBJECTTYPE='RQUSERDEFINEDLISTITEMS') THEN
         -- DELETE THE RQTLOGMAP RECORDS FOR RQUSERDEFINEDLISTITEMS
         DELETE FROM RQTLOGMAP WHERE ID=L_TLOGMAPID;
         
      END IF;

   end if;

END@



-- Update flag for integration completion
UPDATE RQREQUISITE SET RESERVEDFLAGS = UDFBITOR( INTEGER(RESERVEDFLAGS), 2 ) @



